import math
from designer import DesignerObject
import pygame
import designer
import random


class Animation:
    def __init__(self, speed, direction):
        self.speed = speed
        self.direction = direction

    def start(self):
        pass

    def is_finished(self):
        return False


class GlideAnimation(Animation):
    def __init__(self, speed, direction):
        """
        Initializer of animations that glide across the screen.
        Calls initializer of Animations, parent  class.

        :param speed: Pixels to move per second
        :type speed: int
        :param direction: Degrees at which to move
        :type direction: int
        """
        direction = math.radians(direction)
        self.x = math.cos(direction)
        self.y = math.sin(direction + math.pi)

        # handle moving by at least one pixel if value is less than one but not accounting for when value is genuinely 0
        if 0.01 < self.x < 1:
            self.x = math.ceil(self.x)
        if 0.01 < self.y < 1:
            self.y = math.ceil(self.y)
        super().__init__(speed, direction)

    def step(self, object):
        '''
        Handles glide for each step of the game state.

        :param object: Object to move
        :type object: DesignerObject or DesignerGroup
        :return: None
        '''
        if isinstance(object, DesignerObject):
            objects = [object]
        elif isinstance(object, designer.DesignerGroup):
            objects = [object]
        for temp_object in objects:
            if (
                    temp_object.rect.right) < designer.GLOBAL_DIRECTOR.width and temp_object.rect.x > 0 and temp_object.rect.top < designer.GLOBAL_DIRECTOR.height and temp_object.rect.y > 0:
                if not object.finished_animation:
                    temp_object.rect.x += (self.speed * self.x)
                    temp_object.rect.y += (self.speed * self.y)
                    temp_object.dirty = 1
            else:
                object.finished_animation = True


class JitterAnimation(Animation):
    def __init__(self, direction):
        self.direction = direction

    def step(self, object):
        '''
              Handles glide for each step of the game state.

                :param object: Object to move
                :type object: DesignerObject or DesignerGroup
                :return: None
                '''
        if isinstance(object, DesignerObject):
            objects = [object]
        elif isinstance(object, designer.DesignerGroup):
            objects = object.objects
            x_dir = random.randint(-self.direction, self.direction)
            y_dir = random.randint(-self.direction, self.direction)
            for object in objects:
                object.rect.x += x_dir
                object.rect.y += y_dir
                object.dirty = 1


class RotateAnimation(Animation):
    def __init__(self, speed, direction, angle_limit, pos):
        self.speed = speed
        self.angle = 1
        self.angle_limit = angle_limit
        self.pos = pos
        self.total = 0
        super().__init__(speed, direction)

    def step(self, obj):
        '''
        Handles rotation for each step of the game state.

        :param obj: Object to be moved
        :type obj: DesignerObject or DesignerGroup
        :return: None
        '''
        if isinstance(obj, DesignerObject):
            objects = [obj]
        elif isinstance(obj, designer.DesignerGroup):
            objects = obj.objects
        for obj in objects:
            if self.total < self.angle_limit:
                self.total = self.total + (self.speed * self.angle)
                obj.image = pygame.transform.rotate(obj.orig_img, self.total)
                obj.rect = obj.image.get_rect(center=obj.orig_img.get_rect(center=(obj.rect.center)).center)

                obj.dirty = 1


def glide_around(*objects, speed):
    '''
    Moves object around at random.

    :param objects: Object to move around
    :type objects: DesignerObject or DesignerGroup
    :param speed: Pixels to move per second
    :type speed: int
    :return: None
    '''
    for object in objects:
        object.add_animation(JitterAnimation(speed, 0))


def glide_right(obj, speed):
    '''
       Moves object to the right of the window.

       :param obj: Object to move
       :type obj: DesignerObject or DesignerGroup
       :param speed: Pixels to move per second
       :type speed: int
       :return: None
       '''
    obj.add_animation(GlideAnimation(speed, 0))


def glide_left(obj, speed):
    '''
           Moves object to the left of the window.

           :param obj: Object to move
           :type obj: DesignerObject
           :param speed: Pixels to move per second
           :type speed: int
           :return: None
           '''
    obj.add_animation(GlideAnimation(speed, 180))


def glide_up(obj, speed):
    '''
           Moves object up on the window.

           :param obj: Object to move
           :type obj: DesignerObject or DesignerGroup
           :param speed: Pixels to move per second
           :type speed: int
           :return: None
     '''
    obj.add_animation(GlideAnimation(speed, 90))


def glide_down(obj, speed):
    '''
            Moves object down on the window!

            :param obj: Object to move
            :type obj: DesignerObject or DesignerGroup
            :param speed: Pixels to move per second
            :type speed: int
            :return: None
         '''
    obj.add_animation((GlideAnimation(speed, 270)))


def glide_in_degrees(obj, direction, speed):
    '''
    Moves object a given number of degrees in that direction

    :param obj: Object to move
    :type obj: DesignerObject or DesignerGroup
    :param direction: Direction in degrees counterclockwise for object to move
    :type direction: int
    :param speed: Pixels to move per second
    :type speed: int
    :return: None
    '''
    obj.add_animation((GlideAnimation(speed, direction)))


def rotate(obj, angle_limit, speed):
    '''
    Rotates object in place for a given number of degrees

    :param obj: Object to move
    :type obj: DesignerObject or DesignerGroup
    :param angle_limit: Degrees to rotate object
    :type angle_limit: int
    :param speed: Pixels to move per second
    :type speed: int
    :return: None
    '''
    if isinstance(obj, DesignerObject.DesignerObject):
        obj.add_animation(RotateAnimation(speed, 0, angle_limit, obj.rect.topleft))
    if isinstance(obj, DesignerObject.DesignerGroup):
        for temp_object in obj.objects:
            temp_object.add_animation(RotateAnimation(speed, 0, angle_limit, temp_object.rect.topleft))



from typing import List

import designer
import pygame


class Director:
    def __init__(self, width=800, height=600, background_color=(255, 255, 255), fps = 50):
        """
        Initializes the Director that will control the game state.

        :param width: width of the game window in pixels
        :type width: int
        :param height: height of the game window in pixels
        :type height: int
        :param background_color: color to initially fill the window with
        """
        pygame.init()
        self.window_size = width, height
        self.screen = pygame.display.set_mode(self.window_size)
        self.bkgr_color = background_color
        self.screen.fill(background_color)

        self.width = width
        self.height = height
        self.background = pygame.Surface(self.screen.get_size()).convert()
        self.background.fill(self.bkgr_color)

        self.fps = fps
        self.running = False
        self.all_game_objects = pygame.sprite.LayeredDirty()
        self.groups = []


    def start(self):
        """
        Starts Pygame main game loop. Checks for events and DirtySprite updates. Handles animations.

        :return: None
        """
        self.running = True
        self.all_game_objects.clear(self.screen, self.background)
        self.screen = pygame.display.set_mode(self.window_size)
        self.screen.fill(self.bkgr_color)


        time = 0
        while self.running:
            pygame.time.Clock().tick(self.fps)
            for event in pygame.event.get():
                if event.type == pygame.QUIT:
                    self.running = False
            for gobject in self.all_game_objects:
                gobject._handle_animation()
            for group in self.groups:
                group._handle_animation()
            time += 1
            self.all_game_objects.update()
            rects = self.all_game_objects.draw(self.screen)
            pygame.display.update(rects)

    def add(self, *images):
        """
        Adds a DirtySprite to Director's DirtySprite collection.

        :param images: images to add to self's DirtySprite collection
        :type images: designer.DesignerObjects

        :return: nOne
        """
        for image in images:
            self.all_game_objects.add(image)


    def add_group(self, *groups):
        """
        Adds group of DirtySprites to Director's group collection.

        :param groups: groups to add to self's group collection
        :type groups: designer.DesignerGroups

        :return: None
        """
        for group in groups:
            self.groups.append(group)


def check_initialized():
    """
    Checks if global state exists and creates one if it does not.

    :return: None
    """
    if designer.GLOBAL_DIRECTOR is None:
        designer.GLOBAL_DIRECTOR = Director()


def draw(*objs):
    """
    Internally starts the game loop.

    :return: None
    """
    check_initialized()
    designer.GLOBAL_DIRECTOR.start()


def set_window_color(color):
    '''
    Changes window color to given color.
    Must call before adding any DesignerObjects.

    :param color: color to change window to
    :type color: str or List[str]

    :return: None
    '''
    check_initialized()
    designer.GLOBAL_DIRECTOR.bkgr_color = color
    designer.GLOBAL_DIRECTOR.screen.fill(color)
    designer.GLOBAL_DIRECTOR.background.fill(color)

def set_window_size(width, height):
    """
    Set size of window in pixels.
    Must call before adding any DesignerObjects.

    :param width: number of pixels to set horizontal size of window
    :type width: int
    :param height: number of pixels to set vertical size of window
    :type height: int
    :return: None
    """
    check_initialized()
    designer.GLOBAL_DIRECTOR.window_size = width, height



import importlib
import json
import os

with open("../designer/data/" + "colors.json") as f:
    colors = json.load(f)
    f.close()

def _process_color(color) -> 'List[int]':
    """
    Converts either a color string (e.g., 'orange') into a color triplet (list of 3 numbers).
    If a color triplet is passed in, it is returned unmodified.

    :param color:
    :return:
    """
    if isinstance(color, str):
        return colors[color]
    else:
        return color



from os import path

import setuptools

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='designer',
    version='0.0.72',
    python_requires='>=3.6',
    author='krishols, acbart',
    packages=['designer'],
    package_data={'designer': ['data/colors.json']},
    author_email='kris@udel.edu',
    description='Student-friendly and evidence-based visual graphics library.',
    license='MIT',
    long_description=long_description,
    url='https://github.com/krishols/designer',
    classifiers=[
        'Development Status :: 3 - Alpha',

        'Topic :: Education',

        'License :: OSI Approved :: MIT License',

        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ])